/* 
 * File:   DAQChannelMapBase.cpp
 * Author: denis
 * 
 * Created on February 10, 2012, 10:17 AM
 */

#include "DAQChannelMap/DAQChannelMapBase.h"

#include <iostream>

using namespace daqchannelmap;

// Standard enumeration
#define PUT_ENUM(INPUT) CHAN_IDX_0_1                 \
                        INPUT##_NEARDET,             \
                        INPUT##_FARDET,              \
                        INPUT##_NDOS,                \
			INPUT##_TESTBEAM_2Block_1x1, \
			INPUT##_TESTBEAM_3Block_1x1, \
                        INPUT##_TESTBEAM_2Block_2x2, \
                        INPUT##_TESTBEAM_3Block_2x2, \
                        INPUT##_IPND                


const unsigned int DAQChannelMapBase::NUM_XCHAN_PLANE[] = { PUT_ENUM(NUM_CHAN_X_PLANE) };
const unsigned int DAQChannelMapBase::NUM_YCHAN_PLANE[] = { PUT_ENUM(NUM_CHAN_Y_PLANE)};
const unsigned int DAQChannelMapBase::NUM_DIBLOCK_DET[] = { PUT_ENUM(NUM_DIBLOCK) };
const unsigned int DAQChannelMapBase::NUM_DCMS_DIBLOCK[]= { PUT_ENUM(NUM_DCMS_DIBLOCK) };
const unsigned int DAQChannelMapBase::NUM_X_DCMS[]      = { PUT_ENUM(NUM_X_DCMS) };
const unsigned int DAQChannelMapBase::NUM_Y_DCMS[]      = { PUT_ENUM(NUM_Y_DCMS) };


const int DAQChannelMapBase::DetectorDiBlockConfig_IPND[]    = {CHAN_IDX_0_1
                                                                AB_DIBLOCK,AB_DIBLOCK,
                                                                TERM_CHAR
                                                               };

const int DAQChannelMapBase::DetectorDiBlockConfig_NEARDET[] = {CHAN_IDX_0_1
                                                                AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,
                                                                AFULLAFULL_DIBLOCK, // Muon catcher  (actually it's just incomplete FULL-block)
                                                                TERM_CHAR
                                                               };

const int DAQChannelMapBase::DetectorDiBlockConfig_NDOS[]    = {CHAN_IDX_0_1
                                                                AB_DIBLOCK,AB_DIBLOCK,AA_DIBLOCK,
                                                                MC_DIBLOCK, // Muon catcher  (actually it's just incomplete B-block)
                                                                TERM_CHAR
                                                               };

const int DAQChannelMapBase::DetectorDiBlockConfig_FARDET[]  = {CHAN_IDX_0_1
                                                                AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,
                                                                AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,
                                                                AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,
                                                                AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,
                                                                AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,
                                                                TERM_CHAR
                                                               };


const int DAQChannelMapBase::DetectorDiBlockConfig_TESTBEAM_2Block_1x1[] = {CHAN_IDX_0_1
                                                                            AFULLAFULL_DIBLOCK,
                                                                            TERM_CHAR
                                                                           };
const int DAQChannelMapBase::DetectorDiBlockConfig_TESTBEAM_3Block_1x1[] = {CHAN_IDX_0_1
                                                                            AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,
                                                                            TERM_CHAR
                                                                           };
const int DAQChannelMapBase::DetectorDiBlockConfig_TESTBEAM_2Block_2x2[] = {CHAN_IDX_0_1
                                                                            AFULLAFULL_DIBLOCK,
                                                                            TERM_CHAR
                                                                           };
const int DAQChannelMapBase::DetectorDiBlockConfig_TESTBEAM_3Block_2x2[] = {CHAN_IDX_0_1
                                                                            AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,
                                                                            TERM_CHAR
                                                                           };

const int* DAQChannelMapBase::DetectorDiBlockConfig[]    = { PUT_ENUM(DetectorDiBlockConfig),(int*)TERM_CHAR };

// Mapping of APD, index is the Pixel Channel, result is the module cell
const unsigned int DAQChannelMapBase::apd_pixel2cell_v0[] = {25,24,27,26,29,28,31,30,
						             17,16,19,18,21,20,22,23,
						             15,14,12,13,10,11, 8, 9,
						             6, 7, 4, 5, 2, 3, 0, 1
                                                             };
// Back Mapping of APD, index is the module cell, result is the APD channel
const unsigned int DAQChannelMapBase::apd_cell2pixel_v0[] = {30,31,28,29,26,27,24,25,
						      22,23,20,21,18,19,17,16,
						      9, 8,11,10,13,12,14,15,
						      1, 0, 3, 2, 5, 4, 7, 6};

const char* DAQChannelMapBase::DetectorLabels[] = {
  "NULL",
  "NEARDET",
  "FARDET",
  "NDOS",
  "TESTBEAM_1x1x2",
  "TESTBEAM_1x1x3",
  "TESTBEAM_2x2x2",
  "TESTBEAM_2x2x3",
  "Detector Unknown"
  "IPND",
};

//------------------------------------------------------------------------------
DAQChannelMapBase::DAQChannelMapBase() {

    initApdPixel2PixelMap();

    initNumberOfCellsPerDCMInDiblock();

    initBlockOrderInDiblock();

    initPlanesInBlockID();

    initNumberOfTypePlanesInBlock();

    initPlanesInDiblockID();
    
    initStartPlaneTypeInBlock();

    initPlanesInBlockAndDiblockDetector();

    initNumberOfDCMsInDiblockDetector();
}

//------------------------------------------------------------------------------
DAQChannelMapBase::~DAQChannelMapBase() { }


//------------------------------------------------------------------------------
/// Initialization methods
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
bool DAQChannelMapBase::initApdPixel2PixelMap(){

       /// First, need to resize the maps
       _apdPixel2CellMap.resize(32);
       _cell2ApdPixelMap.resize(32);

       const unsigned int*  apd_pixel2cell_pointer = apd_pixel2cell_v0;

       /// First, construct the map of apdPixel2CellMap
       for(uint32_t iapd_pixel = 0; iapd_pixel < 32; ++iapd_pixel){
           const pixel_t ipixel = (pixel_t) (apd_pixel2cell_pointer[iapd_pixel]);
           _apdPixel2CellMap[iapd_pixel] = ipixel;
           _cell2ApdPixelMap[ipixel]     = iapd_pixel;
       }

       return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBase::initNumberOfCellsPerDCMInDiblock(){

    _numberOfCellsPerDCMInDiblock.resize(MAX_DIBLOCK_TYPE+1);
    _numberOfCellsPerDCMInDiblock[AB_DIBLOCK]         = NUM_CELLS_PER_DCM;
    _numberOfCellsPerDCMInDiblock[BA_DIBLOCK]         = NUM_CELLS_PER_DCM;
    _numberOfCellsPerDCMInDiblock[AA_DIBLOCK]         = NUM_CELLS_PER_DCM;
    _numberOfCellsPerDCMInDiblock[AFULLAFULL_DIBLOCK] = NUM_CELLS_PER_DCM;
    _numberOfCellsPerDCMInDiblock[MC_DIBLOCK]         = 2*NUM_CELLS_PER_DCM;

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBase::isinitNumberOfCellsPerDCMInDiblock(){

    /// Needs _numberOfCellsPerDCMInDiblock initialized
    if( _numberOfCellsPerDCMInDiblock.size() < MAX_BLOCK_TYPE+1 && !initNumberOfCellsPerDCMInDiblock() ) {
            std::cerr<<"initNumberOfCellsPerDCMInDiblock can not be initialized\n";
            exit(1);
    }// end of check whether _numberOfCellsPerDCMInDiblock is initialized

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBase::initBlockOrderInDiblock(){

    for(uint32_t idiblock_type = 0; idiblock_type <= MAX_DIBLOCK_TYPE; idiblock_type++){
        std::vector <uint32_t> temp_vector;
        temp_vector.resize(2);
        _BlockOrderInDiblock.push_back(temp_vector);
    }/// end of loop over diblock types

    _BlockOrderInDiblock[AB_DIBLOCK][0]         = A_BLOCK;
    _BlockOrderInDiblock[AB_DIBLOCK][1]         = B_BLOCK;
    _BlockOrderInDiblock[BA_DIBLOCK][0]         = B_BLOCK;
    _BlockOrderInDiblock[BA_DIBLOCK][1]         = A_BLOCK;
    _BlockOrderInDiblock[AA_DIBLOCK][0]         = A_BLOCK;
    _BlockOrderInDiblock[AA_DIBLOCK][1]         = A_BLOCK;
    _BlockOrderInDiblock[AFULLAFULL_DIBLOCK][0] = AFULL_BLOCK;
    _BlockOrderInDiblock[AFULLAFULL_DIBLOCK][1] = AFULL_BLOCK;
    _BlockOrderInDiblock[MC_DIBLOCK][0]         = M_BLOCK;
    _BlockOrderInDiblock[MC_DIBLOCK][1]         = M_BLOCK;

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBase::isinitBlockOrderInDiblock(){

    /// Needs BlockOrderInDiblock initialized
    if( _BlockOrderInDiblock.size() < MAX_DIBLOCK_TYPE+1 && !initBlockOrderInDiblock() ) {
            std::cerr<<"BlockOrderInDiblock can not be initialized\n";
            exit(1);
    }// end of check whether BlockOrderInDiblock is initialized

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBase::initPlanesInBlockID(){

    _planesInBlockType.resize(MAX_BLOCK_TYPE+1);
    _planesInBlockType[A_BLOCK]     = NUM_XZ_PLANES_A_BLOCK     + NUM_YZ_PLANES_A_BLOCK;
    _planesInBlockType[B_BLOCK]     = NUM_XZ_PLANES_B_BLOCK     + NUM_YZ_PLANES_B_BLOCK;
    _planesInBlockType[AFULL_BLOCK] = NUM_XZ_PLANES_AFULL_BLOCK + NUM_YZ_PLANES_AFULL_BLOCK;
    _planesInBlockType[M_BLOCK]     = NUM_XZ_PLANES_MC_BLOCK    + NUM_YZ_PLANES_MC_BLOCK;

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBase::isinitPlanesInBlockID(){

    /// Needs _planesInBlockType initialized
    if( _planesInBlockType.size() < MAX_BLOCK_TYPE+1 && !initPlanesInBlockID() ) {
            std::cerr<<"_planesInBlockType can not be initialized\n";
            exit(1);
    }// end of check whether _planesInBlockType is initialized

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBase::initNumberOfTypePlanesInBlock(){

        isinitPlanesInBlockID();

        numberOfPlanesInBlock_t number_of_yz_planes_in_block;
        numberOfPlanesInBlock_t number_of_xz_planes_in_block;
        number_of_yz_planes_in_block.resize(MAX_BLOCK_TYPE+1);
        number_of_xz_planes_in_block.resize(MAX_BLOCK_TYPE+1);

        number_of_yz_planes_in_block[A_BLOCK]     = NUM_YZ_PLANES_A_BLOCK;     ///< Number of vertical planes. A-Block starts with horizontal plane
        number_of_yz_planes_in_block[B_BLOCK]     = NUM_YZ_PLANES_B_BLOCK;     ///< B-block starts with vertical plane
        number_of_yz_planes_in_block[AFULL_BLOCK] = NUM_YZ_PLANES_AFULL_BLOCK; ///< AFULL-block starts with horizontal plane
        number_of_yz_planes_in_block[M_BLOCK]     = NUM_YZ_PLANES_MC_BLOCK;    ///< Muon catcher starts with vertical plane

        /// Calculating number of XZ planes for each Block Type
        for(block_t iblock_type = MIN_BLOCK_TYPE; iblock_type <= MAX_BLOCK_TYPE; iblock_type++){
            const DetBlock_TYPE block_type           = (DetBlock_TYPE)iblock_type;
            number_of_xz_planes_in_block[block_type] = getNumberOfPlanesInBlock(block_type) - number_of_yz_planes_in_block[block_type];
        }// end of loop over blocks calculating number of XZ planes for each Block Type

        _numberOfTypePlanesInBlock[XZ_PLANE] = number_of_xz_planes_in_block;
        _numberOfTypePlanesInBlock[YZ_PLANE] = number_of_yz_planes_in_block;

        return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBase::isinitNumberOfTypePlanesInBlock(){

    /// Needs _numberOfTypePlanesInBlock initialized
    if( _numberOfTypePlanesInBlock.size() < 2 && !initNumberOfTypePlanesInBlock() ) {
            std::cerr<<"_numberOfTypePlanesInBlock can not be initialized\n";
            exit(1);
    }// end of check whether _numberOfTypePlanesInBlock is initialized

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBase::initPlanesInDiblockID(){


    /// Needs PlanesInBlockID initialized
    isinitPlanesInBlockID();
    /// Needs BlockOrderInDiblock initialized
    isinitBlockOrderInDiblock();
    /// Needs NumberOfTypePlanesInBlock initialized
    isinitNumberOfTypePlanesInBlock();

    /// Filling Number of planes in Diblock and each of its views
    _planesInDiblockType.resize(MAX_DIBLOCK_TYPE+1);

    _numberOfTypePlanesInDiBlock[XZ_PLANE].resize(MAX_DIBLOCK_TYPE+1);
    _numberOfTypePlanesInDiBlock[YZ_PLANE].resize(MAX_DIBLOCK_TYPE+1);
    for(uint32_t idiblock_type = MIN_DIBLOCK_TYPE; idiblock_type <= MAX_DIBLOCK_TYPE; idiblock_type++){
        const DetBlock_TYPE block0 = (DetBlock_TYPE)_BlockOrderInDiblock[idiblock_type][0];
        const DetBlock_TYPE block1 = (DetBlock_TYPE)_BlockOrderInDiblock[idiblock_type][1];

        _planesInDiblockType[idiblock_type] = _planesInBlockType[block0] + _planesInBlockType[block1];
        _numberOfTypePlanesInDiBlock[XZ_PLANE][(DiBlock_TYPE)idiblock_type] = _numberOfTypePlanesInBlock[XZ_PLANE][block0] + _numberOfTypePlanesInBlock[XZ_PLANE][block1];
        _numberOfTypePlanesInDiBlock[YZ_PLANE][(DiBlock_TYPE)idiblock_type] = _numberOfTypePlanesInBlock[YZ_PLANE][block0] + _numberOfTypePlanesInBlock[YZ_PLANE][block1];
    }// end of loop over diblocks filling Number of planes in Diblock and each of its views

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBase::isinitPlanesInDiblockID(){

    /// Needs _numberOfTypePlanesInDiBlock initialized
    if( _numberOfTypePlanesInDiBlock.size() < 2 && !initPlanesInDiblockID() ) {
            std::cerr<<"_numberOfTypePlanesInDiBlock can not be initialized\n";
            exit(1);
    }// end of check whether _numberOfTypePlanesInDiBlock is initialized

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBase::initStartPlaneTypeInBlock(){

    _startPlaneTypeInBlock.resize(MAX_BLOCK_TYPE+1);
    _startPlaneTypeInBlock[A_BLOCK]     = XZ_PLANE; /// A-block starts with horizontal plane
    _startPlaneTypeInBlock[B_BLOCK]     = YZ_PLANE; /// B-block starts with vertical plane
    _startPlaneTypeInBlock[AFULL_BLOCK] = XZ_PLANE; /// AFULL-block starts with horizontal plane
    _startPlaneTypeInBlock[M_BLOCK]     = YZ_PLANE; /// M-block starts with vertical plane

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBase::isinitStartPlaneTypeInBlock(){

    /// Needs _startPlaneTypeInBlock initialized
    if( _startPlaneTypeInBlock.size() < MAX_BLOCK_TYPE+1 && !initStartPlaneTypeInBlock() ) {
            std::cerr<<"_startPlaneTypeInBlock can not be initialized\n";
            exit(1);
    }// end of check whether _startPlaneTypeInBlock is initialized

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBase::initPlanesInBlockAndDiblockDetector(){


    /// Needs PlanesInBlockID initialized
    isinitPlanesInBlockID();
    /// Needs BlockOrderInDiblock initialized
    isinitBlockOrderInDiblock();


    /// Fill Number of Planes in Diblock
    for(uint32_t idet = 0; idet <= MAX_DET_ID; idet++){
        std::vector <uint32_t> number_of_planes_in_diblock_for_current_detector;
        std::vector <uint32_t> number_of_planes_in_block_for_current_detector;

        /// Fill stuff only for legal detectors, otherwise leave vectors empty
        if(idet >= MIN_DET_ID){

            // we index from 1 on diblocks and blocks, so need to push 0
            number_of_planes_in_block_for_current_detector.push_back(0);
            number_of_planes_in_diblock_for_current_detector.push_back(0);

            /// Loop over all diblock numbers in the detector
            for(diblock_t idiblock = 1; idiblock <= MAX_DIBLOCK; idiblock++){

                const DiBlock_TYPE diblock_type = getDiBlockTypeBase((DetectorIdent)idet, idiblock);

                if(diblock_type == (int)TERM_CHAR) break;

                /// Fill planes for diblock
                number_of_planes_in_diblock_for_current_detector.push_back(getNumberOfPlanesInDiblock(diblock_type));

                /// Blocks in the current diblock
                const DetBlock_TYPE block0 = (DetBlock_TYPE)_BlockOrderInDiblock[diblock_type][0];
                const DetBlock_TYPE block1 = (DetBlock_TYPE)_BlockOrderInDiblock[diblock_type][1];

                /// Fill planes for two blocks of the current diblock
                number_of_planes_in_block_for_current_detector.push_back(_planesInBlockType[block0]);
                number_of_planes_in_block_for_current_detector.push_back(_planesInBlockType[block1]);
            }// end of loop over diblocks
        }// end of if detector is legal

        _planesInDiblockDetector.push_back(number_of_planes_in_diblock_for_current_detector);
        _planesInBlockDetector.push_back(number_of_planes_in_block_for_current_detector);
    }// end of loop over detectors

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBase::isinitPlanesInBlockAndDiblockDetector(){

    /// Needs _planesInBlockDetector initialized
    if( _planesInBlockDetector.size() < MAX_DET_ID && !initPlanesInBlockAndDiblockDetector() ) {
            std::cerr<<"_planesInBlockDetector can not be initialized\n";
            exit(1);
    }// end of check whether _planesInBlockDetector is initialized

    return true;
}


//------------------------------------------------------------------------------
bool DAQChannelMapBase::initNumberOfDCMsInDiblockDetector(){

    /// Initialize vectors
    for(uint32_t detector_id = 0; detector_id <= MAX_DET_ID; detector_id++){
        std::vector<uint32_t> tmp_vector;
        tmp_vector.resize(MAX_DIBLOCK_TYPE+1);

        _numberOfXDCMsInDiblockDetector.push_back(tmp_vector);
        _numberOfYDCMsInDiblockDetector.push_back(tmp_vector);
    }// end of initializing vectors


    for(uint32_t  detector = MIN_DET_ID; detector <= MAX_DET_ID; detector++){
        for(diblock_t idiblock = MIN_DIBLOCK_TYPE; idiblock <= MAX_DIBLOCK_TYPE; idiblock++){
        _numberOfXDCMsInDiblockDetector[detector][idiblock] = NUM_X_DCMS[detector];
        _numberOfYDCMsInDiblockDetector[detector][idiblock] = NUM_Y_DCMS[detector];
        }
    }// end of filling number of DCMs


    /// Exception is the muon catcher, for which number of DCMs is 1 in each view
    for(uint32_t  detector = MIN_DET_ID; detector <= MAX_DET_ID; detector++){
        _numberOfXDCMsInDiblockDetector[detector][MC_DIBLOCK] = 1;
        _numberOfYDCMsInDiblockDetector[detector][MC_DIBLOCK] = 1;
    }// end of filling exceptions


    return true;
}// end of initNumberOfXDCMsInDiblockDetector


//------------------------------------------------------------------------------
bool DAQChannelMapBase::isinitNumberOfDCMsInDiblockDetector(){

    /// Needs _numberOfXDCMsInDiblockDetector initialized
    if( _numberOfXDCMsInDiblockDetector.size() < MAX_DET_ID && !initNumberOfDCMsInDiblockDetector() ) {
            std::cerr<<"_numberOfXDCMsInDiblockDetector can not be initialized\n";
            exit(1);
    }// end of check whether _numberOfXDCMsInDiblockDetector is initialized

    return true;
}

//------------------------------------------------------------------------------
pixel_t       DAQChannelMapBase::APDPixel2Pixel(apd_pixel_t apd_pixel) const  {return _apdPixel2CellMap[apd_pixel];}
apd_pixel_t   DAQChannelMapBase::pixel2APDPixel(pixel_t     pixel)     const {return _cell2ApdPixelMap[pixel];}


DiBlock_TYPE  DAQChannelMapBase::getDiBlockTypeBase        (DetectorIdent detector, diblock_t diblock_number) const {return (DiBlock_TYPE)DetectorDiBlockConfig[detector][diblock_number];}
uint32_t      DAQChannelMapBase::getNumberOfPlanesInBlock  (DetectorIdent detector, block_t   block)   const {return _planesInBlockDetector[detector][block];}
uint32_t      DAQChannelMapBase::getNumberOfPlanesInDiblock(DetectorIdent detector, diblock_t diblock) const {return _planesInDiblockDetector[detector][diblock];}
uint32_t      DAQChannelMapBase::getNumberOfPlanesInBlock  (DetBlock_TYPE block_type  ) const {return _planesInBlockType[block_type];}
uint32_t      DAQChannelMapBase::getNumberOfPlanesInDiblock(DiBlock_TYPE  diblock_type) const {return _planesInDiblockType[diblock_type];}

uint32_t      DAQChannelMapBase::getBlockNumber   (DetectorIdent detector, offline_plane_t plane_0) const {return getBlockOrDiblock(detector, plane_0, _planesInBlockDetector);}
uint32_t      DAQChannelMapBase::getDiblockNumber (DetectorIdent detector, offline_plane_t plane_0) const {return getBlockOrDiblock(detector, plane_0, _planesInDiblockDetector);}
uint32_t      DAQChannelMapBase::getNumberOfXDCM(DetectorIdent detector, DiBlock_TYPE  diblock_type) const{return _numberOfXDCMsInDiblockDetector[detector][diblock_type];}
uint32_t      DAQChannelMapBase::getNumberOfYDCM(DetectorIdent detector, DiBlock_TYPE  diblock_type) const{return _numberOfYDCMsInDiblockDetector[detector][diblock_type];}
uint32_t      DAQChannelMapBase::getNumberOfDCM (DetectorIdent detector, DiBlock_TYPE  diblock_type) const {return getNumberOfXDCM(detector, diblock_type) + getNumberOfYDCM(detector, diblock_type);}
uint32_t      DAQChannelMapBase::getNumberOfChannelsInXPlane(DetectorIdent detector) const{return NUM_XCHAN_PLANE[detector];}
uint32_t      DAQChannelMapBase::getNumberOfChannelsInYPlane(DetectorIdent detector) const{return NUM_YCHAN_PLANE[detector];}

uint32_t      DAQChannelMapBase::getNumberOfPlaneTypeInBlock  (DetBlock_TYPE block_type  , DetPlane_TYPE plane_type) {return _numberOfTypePlanesInBlock[plane_type][block_type];}
uint32_t      DAQChannelMapBase::getNumberOfPlaneTypeInDiblock(DiBlock_TYPE  diblock_type, DetPlane_TYPE plane_type) {return _numberOfTypePlanesInDiBlock[plane_type][diblock_type];}

DetBlock_TYPE DAQChannelMapBase::getBlockTypeBase(DiBlock_TYPE diblock_type, uint32_t local_block_number_in_diblock) const{return (DetBlock_TYPE)_BlockOrderInDiblock[diblock_type][local_block_number_in_diblock];}

DetPlane_TYPE DAQChannelMapBase::getStartPlaneTypeInBlock(DetBlock_TYPE block_type) const {return _startPlaneTypeInBlock[block_type];}

uint32_t      DAQChannelMapBase::getNumberOfCellsPerDCM(DiBlock_TYPE diblock_type) const{ return _numberOfCellsPerDCMInDiblock[diblock_type];}

uint32_t      DAQChannelMapBase::getTotalNumberOfBlocks(DetectorIdent detector) const { return 2* getTotalNumberOfDiblocks(detector);}
uint32_t      DAQChannelMapBase::getTotalNumberOfDiblocks(DetectorIdent detector) const { return _planesInDiblockDetector[detector].size() - 1;}


//------------------------------------------------------------------------------
std::string   DAQChannelMapBase::getDetectorLabel(DetectorIdent detector) const {
    const std::string detector_label = DetectorLabels[detector];

    return detector_label;
}

//------------------------------------------------------------------------------
uint32_t DAQChannelMapBase::getBlockOrDiblock(DetectorIdent detector, offline_plane_t plane_0,
                                              const numberOfPlanesInBlockDetector_t& planes_in_block_or_diblock) const {

  CHECKERROR( plane_0);

  offline_plane_t planescount = 0;
  uint32_t iblock_or_diblock  = 1; //we index from 1 on diblocks and blocks
  for(; iblock_or_diblock < planes_in_block_or_diblock[detector].size(); iblock_or_diblock++) {
      planescount += planes_in_block_or_diblock[detector][iblock_or_diblock];
      if(plane_0 < planescount) return iblock_or_diblock;
  }

  return iblock_or_diblock;
}

//------------------------------------------------------------------------------
// Returns local plane number for the diblock from offline channel
offline_plane_t DAQChannelMapBase::getLocalPlaneInDiblockBase(DetectorIdent detector, offline_plane_t plane_0) const {
  return getLocalPlaneInBlockOrDiblock(detector, plane_0, _planesInDiblockDetector);
}

//------------------------------------------------------------------------------
offline_plane_t DAQChannelMapBase::getLocalPlaneInBlockBase(DetectorIdent detector, offline_plane_t plane_0) const {
  return getLocalPlaneInBlockOrDiblock(detector, plane_0, _planesInBlockDetector);
}

//------------------------------------------------------------------------------
offline_plane_t DAQChannelMapBase::getLocalPlaneInBlockOrDiblock(DetectorIdent detector, offline_plane_t plane_0,
                                                                 const numberOfPlanesInBlockDetector_t& planes_in_block_or_diblock) const {

  offline_plane_t planescount = 0;
  /// Add all planes before the current diblock
  /// we index from 1 on diblocks
  const uint32_t max_number_of_blocks_or_diblocks = planes_in_block_or_diblock[detector].size();
  for(uint32_t iblock_or_diblock = 1; iblock_or_diblock < max_number_of_blocks_or_diblocks; iblock_or_diblock++){

      uint32_t adding = planes_in_block_or_diblock[detector][iblock_or_diblock];

      if(planescount+adding > plane_0) break;

      planescount += adding;
  }// end of loop over blocks or diblocks


  return plane_0 - planescount;
}

//------------------------------------------------------------------------------
DetView_TYPE DAQChannelMapBase::detPlaneToDetView(DetPlane_TYPE detector_plane_type) const {
    if(detector_plane_type == XZ_PLANE)return X_VIEW;
    return Y_VIEW;
}

//------------------------------------------------------------------------------
DetPlane_TYPE DAQChannelMapBase::detViewToDetPlane(DetView_TYPE view_type) const {
    if(view_type == X_VIEW)return XZ_PLANE;
    return YZ_PLANE;
}

//------------------------------------------------------------------------------
DetView_TYPE DAQChannelMapBase::getOtherView(DetView_TYPE input_view) const {
    if(input_view == X_VIEW)return Y_VIEW;
    return X_VIEW;
}

//------------------------------------------------------------------------------
DetPlane_TYPE  DAQChannelMapBase::getOtherPlaneType(DetPlane_TYPE input_plane_type) const {
    if(input_plane_type == XZ_PLANE)return YZ_PLANE;
    return XZ_PLANE;
}

//------------------------------------------------------------------------------
const char* DAQChannelMapBase::errorString(DAQChanMapErrorCodes err_code) const {
  switch(err_code){
  case  ERROR_GENERAL:              return "General Error"; break;
  case  ERROR_UNKNOWN:              return "Unknown Error"; break;
  case  ERROR_UNDEFINED:            return "Undefined Error"; break;
  case  ERROR_CHAN_OUTRANGE:        return "Channel Out of Range"; break;
  case  ERROR_PLANE_OUTRANGE:       return "Plane Out of Range"; break;
  case  ERROR_BLOCK_OUTRANGE:       return "Block Out of Range"; break;
  case  ERROR_DIBLOCK_OUTRANGE:     return "DiBlock Out of Range"; break;
  case  ERROR_DCM_OUTRANGE:         return "DCM Out of Range"; break;
  case  ERROR_PORT_OUTRANGE:        return "Port Out of Range"; break;
  case  ERROR_PIXEL_OUTRANGE:       return "Pixel Out of Range"; break;
  case  ERROR_CELL_OUTRANGE:        return "Cell Out of Range"; break;
  case  ERROR_MODULE_OUTRANGE:      return "Module Out of Range"; break;
  case  ERROR_UNKNOWN_DIBLOCK:      return "Unknown Diblock"; break;
  case  ERROR_UNKNOWN_DIBLOCK_TYPE: return "Unknown Diblock Type"; break;
  case  ERROR_UNKNOWN_DCM:          return "Unknown DCM"; break;
  case  ERROR_UNKNOWN_LINK:         return "Unknown Link"; break;
  case  ERROR_UNKNOWN_VIEW:         return "Unknown Readout View"; break;
  case  ERROR_UNKNOWN_DET:          return "Unknown Detector"; break;
  case  ERROR_UNKNOWN_CELL:         return "Unknown Cell"; break;

  default:
    return "Error Code Not Found";
    break;
  }

  return "Error with Error";
}
